//  Package Tracker 2.1.5
//  Copyright (c) 2005 Monkey Business Labs
//  http://www.monkeybusinesslabs.com/

// Define classes to abstract the various tracking interfaces

// There's only one outstanding request at a time, and I feel safer
// making it a global right now.
var xmlReq;

// Hide the class names behind the generic constructor
function TrackerView(whichTracker) {
    switch (whichTracker) {
        case "Select": return(new SelectTracker());
        case "FedEx" : return(new FedExTracker());
        case "UPS"   : return(new UPSTracker());
        case "DHL"   : return(new DHLTracker());
        default      : throw new Error("Unknown tracker type");
    }
}

// ***** Methods common to any tracker *****

// Constructor
function AnyTracker() {
    this.trackerName = "Unknown";
    
    return this;
}

// These prototype functions could be defined anonymously, but it confuses
// my editor.  More significant things have been done for worse reasons.
function AnyTracker_onShow() {
    updateStatus();
}
AnyTracker.prototype.onShow = AnyTracker_onShow;

// Generate an image path based on the current tracker
function AnyTracker_imagePath(fileName) {
    return("images/" + this.trackerName + "/" + fileName);
}
AnyTracker.prototype.imagePath = AnyTracker_imagePath;

// Wrap an imagePath in a "url()" because we need that a lot for CSS.
function AnyTracker_URLPath(fileName) {
    return("url(" + this.imagePath(fileName) + ")");
}
AnyTracker.prototype.URLPath = AnyTracker_URLPath;

function AnyTracker_init() {
    setActiveStyleSheet(this.trackerName);
}
AnyTracker.prototype.init = AnyTracker_init;

// Called right before the tracker is changed
function AnyTracker_goAway() {
    // stub
}
AnyTracker.prototype.goAway = AnyTracker_goAway;

// Send the request for the package status page
function AnyTracker_requestStatus() {
    // Abort any previous request
    if (xmlReq)
        xmlReq.abort();
    
    // Initialize the XMLHttpRequest object
    xmlReq = new XMLHttpRequest();
    var myThis = this;
    xmlReq.onreadystatechange = function () { myThis.processReqChange() };
    
    // Send the request
    // Try to work around caching bug(?) in Dashboard
    var timeStamp = (new Date).getTime();
    xmlReq.open("GET", this.trackingUrl(trackingNumber) + "&cache_timestamp=" + timeStamp, true);
    xmlReq.send(null);
}
AnyTracker.prototype.requestStatus = AnyTracker_requestStatus;

// Called as the request is processed
function AnyTracker_processReqChange() {
    // We only care when it's finished loading (state 4)
    //alert("readyState = " + xmlReq.readyState);

    if (xmlReq.readyState == 4) {
        // only if "OK"
        if (xmlReq.status == 200) {
            // Pass it off to the response handler
            this.processResponse(xmlReq);
        } else {
            setMessage("widget-status", "Failed to get status: " + xmlReq.statusText + " (" + xmlReq.status + ")");
            // Try to reset in the hopes it will work again later
            xmlReq.abort();
            xmlReq = null;
        }
    }
}
AnyTracker.prototype.processReqChange = AnyTracker_processReqChange;

function AnyTracker_processResponse(xmlReqResponse) {
    // Create a new HTML document to hold the response page
    var response_doc = document.implementation.createHTMLDocument("");
    response_doc.open();
    // Put the result into it so we can look at it
    response_doc.write(xmlReqResponse.responseText);
    // Do the ugly processing part
    this.scrapeHtml(response_doc);
    // Free the XMLHttpRequest object
    xmlReq = null;
}
AnyTracker.prototype.processResponse = AnyTracker_processResponse;

// Activate the warm fuzzy glow when the package is delivered
function AnyTracker_setDelivered(isDelivered) {
    var newClass = isDelivered ? "delivered" : "undelivered";

    document.getElementById("front").className = newClass;
}
AnyTracker.prototype.setDelivered = AnyTracker_setDelivered;

// This function was really clever, so I'm leaving it here.  Also, it can crash
// Dashboard, which is also cool.  But in the end, I decided not to do it this way.
//    // I really wish there was an interface to get a style by name.
//    // So we make one.
//    var styles = new Object();
//    var sheets = document.styleSheets;
//    for (var s = 0; s < sheets.length; s++) {
//      var rules = sheets[s].cssRules;
//      for (var r = 0; r < rules.length; r++) {
//          var rule = rules[r];
//          if (rule.type == 1 /* STYLE_RULE */) {
//              styles[rule.selectorText] = rule.style;
//          }
//      }
//    }
//
//    // Now fly the appropriate colors
//    // Do this with an @imported style sheet and it crashes!
//    styles['*[ID"front"]'].background = this.URLPath("front.png");
//    styles['*[ID"drawer-top"]'].background = this.URLPath("drawer_middle.png");
//    styles['*[ID"drawer-middle"]'].background = this.URLPath("drawer_middle.png");
//    styles['*[ID"drawer-bottom"]'].background = this.URLPath("drawer_bottom.png");
//    styles['.drawer-row-odd'].background = this.URLPath("drawer_middle.png");
//    styles['.drawer-row-even'].background = this.URLPath("drawer_hilight.png");


// ***** Tracker Selector *****
// Inheritance
SelectTracker.prototype = new AnyTracker();
SelectTracker.superClass = AnyTracker.prototype;
SelectTracker.switchedOut = false;

// Constructor
function SelectTracker() {
    this.trackerName = "Select";

    return this;
}

function SelectTracker_init() {
    if (SelectTracker.switchedOut) {
        // Make sure drawer is closed
        closeDrawer();
        // And all the data is emptied
        resetDisplay();
        document.getElementById("trackingField").value = "";
        trackingNumber = null;

        // Un-hide layer
        document.getElementById("service-selector").style.display = "block";
        // Ramp back up to full opacity
        var myThis = this;
        var fadeAnimator = new Animator(myThis.fadeTo, 0);
        fadeAnimator.animateTo(100, 250, myThis.revealDone);
        SelectTracker.switchedOut = false;
    }
}
SelectTracker.prototype.init = SelectTracker_init;

function SelectTracker_onShow() {
    // Override AnyTracker since there's nothing to do when we come onscreen
}
SelectTracker.prototype.onShow = SelectTracker_onShow;

function SelectTracker_fadeTo(fade) {
    document.getElementById("service-selector").style.opacity = fade / 100;
    document.getElementById("front").style.opacity = (100 - fade) / 100;
}
SelectTracker.prototype.fadeTo = SelectTracker_fadeTo;

function SelectTracker_fadeDone() {
    // Turn off our display
    document.getElementById("service-selector").style.display = "none";
    document.getElementById("service-selector").style.opacity = 0.0;
}
SelectTracker.prototype.fadeDone = SelectTracker_fadeDone;

function SelectTracker_revealDone() {
    // Hide the front layer; reset opacity to 100%
    document.getElementById("front").style.display = "none";
    document.getElementById("service-selector").style.opacity = 1.0;
}
SelectTracker.prototype.revealDone = SelectTracker_revealDone;

function SelectTracker_goAway(event) {
    // Hide our layer
    document.getElementById("front").style.display = "block";
    var multiplier = event.shiftKey ? 10 : 1;
    var myThis = this;
    var fadeAnimator = new Animator(myThis.fadeTo, 100);
    fadeAnimator.animateTo(0, 250 * multiplier, myThis.fadeDone);
    SelectTracker.switchedOut = true;
}
SelectTracker.prototype.goAway = SelectTracker_goAway;

// ***** FedEx *****

// Inheritance
FedExTracker.prototype = new AnyTracker();

// Constructor
function FedExTracker() {
    this.trackerName = "FedEx";

    return this;
}

// Get the status for a multiple-piece item
function FedExTracker_requestMpStatus() {
    // Abort any previous request
    if (xmlReq)
        xmlReq.abort();
    
    // Initialize the XMLHttpRequest object
    xmlReq = new XMLHttpRequest();
    var myThis = this;
    xmlReq.onreadystatechange = function () { myThis.processReqChange() };
    
    // Send the request
    // Try to work around caching bug(?) in Dashboard
    var timeStamp = (new Date).getTime();
    xmlReq.open("GET", "http://www.fedex.com/Tracking/Detail?language=english&trackNum=" + trackingNumber + "&cache_timestamp=" + timeStamp, true);
    xmlReq.send(null);
}
FedExTracker.prototype.requestMpStatus = FedExTracker_requestMpStatus;

function FedExTracker_trackingUrl(trackingNumber) {
    return("http://www.fedex.com/Tracking?action=track&tracknumber_list=" + trackingNumber);
}
FedExTracker.prototype.trackingUrl = FedExTracker_trackingUrl;

// Dig through the DOM of the response page and try to pull out the stuff we're interested in
function FedExTracker_scrapeHtml(doc) {
    // If any of this fails, we just bail out
    try {
        // Try to detect and handle multiple-piece numbers
        if (doc.body.innerHTML.indexOf("Multiple-Piece Summary") >= 0) {
            // Queue a request for the details page
            var myThis = this;
            setTimeout(function () { myThis.requestMpStatus() }, 250);
            return;
        }
        
        // Get the table with the information in it
        var infoTable = doc.body.getElementsByTagName("table")[9];

        // Read all the status items into an associative array
        var statusItems = new Object();
        for (var tableIdx = 0; tableIdx < 3; tableIdx++) {
            var rowList = infoTable.getElementsByTagName("table")[tableIdx].getElementsByTagName("tr");
        
            for (var row = 0; row < rowList.length; row++) {
                var cells = rowList[row].getElementsByTagName("td");
                // [label] [spacer] [value] [spacer]
                if (cells.length >= 3) {
                    var labelText = cleanCell(cells[0]);
                    var valueText = cleanCell(cells[2]);
                    if (labelText != null) statusItems[labelText] = valueText;
                }
            }
        }
        
        // Sanity check: If there was no tracking number, give up.
        if (statusItems["Tracking number"] == null)
            throw new Error("Tracking number not found");
        
        var isDelivered = true;
        var estimate = statusItems["Delivery date"];
        if (estimate == null) {
            estimate = statusItems["Estimated delivery"];
            isDelivered = false;
        }
        
        // Update the display
        //setMessage("track-num", statusItems["Tracking number"]);
        resetDisplay();
        this.setDelivered(isDelivered);
        setMessage("estimate", labeledValue(estimateLabel, estimate));
        setMessage("current", labeledValue(currentLabel, statusItems["Status"]));
        
        var activityTable = doc.body.getElementsByTagName("table")[13];
        var rowList = activityTable.getElementsByTagName("tr");

        var numSections = 0;
        drawerSections.length = 0;
        var seenDate = null;
        for (var row = 2; row < rowList.length && numSections < maxRows; row++) {
            var cells = rowList[row].getElementsByTagName("td");
            // XXX - document row format
            if (cells.length >= 14) {
                var dateText     = cleanCell(cells[1]);
                var timeText     = cleanCell(cells[2]);
                var activityText = cleanCell(cells[6]);
                var locationText = cleanCell(cells[10]);
                var detailsText  = cleanCell(cells[14]);

                if (dateText != null) seenDate = dateText;
                var dateTime = timeText;
                if (seenDate != null) dateTime = seenDate + " " + dateTime;
                var line2Text = locationText;
                if (dateTime != null) {
                    if (line2Text != null)
                        line2Text += ", " + dateTime;
                    else
                        line2Text = dateTime;
                }
                numSections++;
                drawerSections.push(labeledValue(activityText, null) + "<br/>"
                                  + labeledValue(null, line2Text));
            }
        }
        setDrawerContents();
    }
    catch (ex) {
        // Exception; presumably the HTML structure wasn't what we expected.
        // Probably bad tracking number or the page layout changed.
        closeDrawer();
        resetDisplay();
        setMessage("widget-status", "Couldn't read package status.");
        setMessage("error", "Tracking number may be incorrect, or the tracking site layout changed.");
    }
}
FedExTracker.prototype.scrapeHtml = FedExTracker_scrapeHtml;


// ***** UPS *****

// Inheritance
UPSTracker.prototype = new AnyTracker();
UPSTracker.superClass = AnyTracker.prototype;

// Constructor
function UPSTracker() {
    this.trackerName = "UPS";
    this.requestPhase = 0;

    return this;
}

function UPSTracker_trackingUrl(trackingNumber) {
    return("http://wwwapps.ups.com/WebTracking/processInputRequest?sort_by=status&TypeOfInquiryNumber=T&InquiryNumber1=" + trackingNumber);
}
UPSTracker.prototype.trackingUrl = UPSTracker_trackingUrl;

function UPSTracker_requestStatus() {
    this.requestPhase = 1;
    UPSTracker.superClass.requestStatus.call(this);
}
UPSTracker.prototype.requestStatus = UPSTracker_requestStatus;

function UPSTracker_processResponse(xmlReqResponse) {
    switch (this.requestPhase) {
        case 1:
            this.requestPhase = 0;
            this.processPhase1(xmlReqResponse);
            break;
        case 2:
            this.requestPhase = 0;
            UPSTracker.superClass.processResponse.call(this, xmlReqResponse);
            break;
    }
}
UPSTracker.prototype.processResponse = UPSTracker_processResponse;

// POST a second request to get the details
function UPSTracker_processPhase1(xmlReqResponse) {
    // Create a new HTML document to hold the response page
    var response_doc = document.implementation.createHTMLDocument("");
    response_doc.open();
    // Put the result into it so we can look at it
    response_doc.write(xmlReqResponse.responseText);
    // Find the form we need to POST to
    // If any of this fails, we just bail out
    try {
        var postForm = response_doc.body.getElementsByTagName("form")[2];
        var postUrl  = postForm.action;
        // Collect the hidden form variables into a POSTable string
        var postBody = "tdts1.x=26&tdts1.y=10"; // Phony click position
        var inputList = postForm.getElementsByTagName("input");
        for (i = 0; i < inputList.length; i++) {
            var inputElement = inputList[i];
            if (postBody.length) postBody += "&";
            postBody += encodeURIComponent(inputElement.name) + "="
                      + encodeURIComponent(inputElement.value);
        }
    }
    catch (ex) {
        // Exception; presumably the HTML structure wasn't what we expected.
        // Probably bad tracking number or the page layout changed.
        closeDrawer();
        resetDisplay();
        setMessage("widget-status", "Couldn't read package status.");
        setMessage("error", "Tracking number may be incorrect, or the tracking site layout changed.");
	alert("UPS Phase1 FAIL: " + xmlReqResponse.responseText);
    }

    // Now generate and POST a new XMLHttpRequest
    if (xmlReq)
        xmlReq.abort();

    this.requestPhase = 2;
    
    // Initialize the XMLHttpRequest object
    xmlReq = new XMLHttpRequest();
    var myThis = this;
    xmlReq.onreadystatechange = function () { myThis.processReqChange() };
    
    // Send the request
    var timeStamp = (new Date).getTime();
    xmlReq.open("POST", postUrl + "?cache_timestamp=" + timeStamp, true);
    xmlReq.setRequestHeader("Content-Type",
                            "application/x-www-form-urlencoded; charset=UTF-8");
    xmlReq.send(postBody);
}
UPSTracker.prototype.processPhase1 = UPSTracker_processPhase1;

function UPSTracker_scrapeHtml(doc) {
    // If any of this fails, we just bail out
    try {
        // Get the table with the information in it
        var infoTable = doc.body.getElementsByTagName("table")[17];

        // Read all the status items into an associative array
        var statusItems = new Object();
        var rowList = infoTable.getElementsByTagName("tr");
    
        for (var row = 0; row < rowList.length; row++) {
            var cells = rowList[row].getElementsByTagName("td");
            // [spacer] [label] [spacer] [value] [spacer]
            if (cells.length >= 3) {
                var labelText = cleanCell(cells[1]);
                var valueText = cleanCell(cells[3]);
                if (labelText != null) statusItems[labelText] = valueText;
            }
        }
        
        // Sanity check: If there was no tracking number, give up.
        if (statusItems["Tracking Number:"] == null)
            throw new Error("Tracking number not found");
        
        // Update the display
        //setMessage("track-num", statusItems["Tracking Number:"]);
        var isDelivered = true;
        var estimate = statusItems["Delivered on:"];
        if (estimate == null) {
            isDelivered = false;
            estimate = statusItems["Scheduled Delivery:"];
        }
        if (estimate == null) {
            isDelivered = false;
            estimate = statusItems["Rescheduled Delivery:"];
        }

        resetDisplay();
        this.setDelivered(isDelivered);
        setMessage("estimate", labeledValue(estimateLabel, estimate));
        setMessage("current", labeledValue(currentLabel, statusItems["Status:"]));

        var activityTable = doc.body.getElementsByTagName("table")[19];
        var rowList = activityTable.getElementsByTagName("tr");
        
        var numSections = 0;
        drawerSections.length = 0;
        for (var row = 4; row < rowList.length && numSections < maxRows; row++) {
            var cells = rowList[row].getElementsByTagName("td");
            // [date/time] [spacer] [location] [spacer] [activity]
            if (cells.length >= 5) {
                var dateTime     = cleanCell(cells[0]);
                var locationText = cleanCell(cells[2]);
                var activityText = cleanCell(cells[4]);
        
                var line2Text = locationText;
                if (dateTime != null) {
                    if (line2Text != null)
                        line2Text += ", " + dateTime;
                    else
                        line2Text = dateTime;
                }
                numSections++;
                drawerSections.push(labeledValue(activityText, null) + "<br/>"
                                  + labeledValue(null, line2Text));
            }
        }
        setDrawerContents();
    }
    catch (ex) {
        // Exception; presumably the HTML structure wasn't what we expected.
        // Probably bad tracking number or the page layout changed.
        closeDrawer();
        resetDisplay();
        setMessage("widget-status", "Couldn't read package status.");
        setMessage("error", "Tracking number may be incorrect, or the tracking site layout changed.");
	//alert("UPS Phase2 FAIL " + ex); // + doc.body.innerHTML);
    }
}
UPSTracker.prototype.scrapeHtml = UPSTracker_scrapeHtml;

// ***** DHL *****

// Inheritance
DHLTracker.prototype = new AnyTracker();

// Constructor
function DHLTracker() {
    this.trackerName = "DHL";

    return this;
}

function DHLTracker_trackingUrl(trackingNumber) {
    return("http://track.dhl-usa.com/TrackByNbr.asp?ShipmentNumber=" + trackingNumber);
}
DHLTracker.prototype.trackingUrl = DHLTracker_trackingUrl;

// Dig through the DOM of the response page and try to pull out the stuff we're interested in
function DHLTracker_scrapeHtml(doc) {
    // If any of this fails, we just bail out
    try {
        // Get the table with the information in it
        var infoTable = doc.body.getElementsByTagName("table")[15];

        // Read all the status items into an associative array
        var statusItems = new Object();
        var rowList = infoTable.getElementsByTagName("table")[0].getElementsByTagName("tr");
    
        for (var row = 0; row < rowList.length; row++) {
            var cells = rowList[row].getElementsByTagName("td");
            // [label] [value]
            if (cells.length >= 2) {
                var labelText = cleanCell(cells[0]);
                var valueText = cleanCell(cells[1]);
                //alert("[" + labelText + "] = [" + valueText + "]");
                if (labelText != null) statusItems[labelText] = valueText;
            }
        }

        // Sanity check: If there was no tracking number, give up.
        //if (statusItems["Tracking number"] == null)
        //    throw new Error("Tracking number not found");
        
        var isDelivered = true;
        var estimate = statusItems["Delivered on"];
        if (estimate == null) {
            isDelivered = true;
            estimate = statusItems["Tendered on"];
        }
        if (estimate == null) {
            isDelivered = false;
            estimate = statusItems["Est. Delivery Date:"];
        }

        var currentStatus = statusItems["Current Status"];
        // XXX godawful hackery
        if (currentStatus)
            currentStatus = currentStatus.replace(/View Signature/, "");

        // Update the display
        resetDisplay();
        this.setDelivered(isDelivered);
        setMessage("estimate", labeledValue(estimateLabel, estimate));
        setMessage("current", labeledValue(currentLabel, currentStatus));
        
        // Find the next table, which should be the scan activity
        var activityTable = infoTable.nextSibling;
        while (activityTable && activityTable.nodeName.toLowerCase() != "table") {
            activityTable = activityTable.nextSibling;
        }
        // Look below the "Tracking history" heading
        var row = activityTable.getElementsByTagName("tr")[0];
        while (row) {
            if (row.nodeName.toLowerCase() == "tr" && row.innerHTML.indexOf("Tracking history") >= 0)
                break;
            row = row.nextSibling;
        }
        row = row.nextSibling.getElementsByTagName("table")[0].getElementsByTagName("tr")[0];

        var numSections = 0;
        drawerSections.length = 0;
        var seenDate = null;
        while ((row = row.nextSibling) && numSections < maxRows) {
            if (row.nodeName.toLowerCase() != "tr")
                continue;
            var cells = row.childNodes;
            // 0[date] 1[status] 2[location]
            if (cells.length >= 3) {
                var dateTime     = cleanCell(cells[0]);
                var activityText = cleanCell(cells[1]);
                var locationText = cleanCell(cells[2]);

                var line2Text = locationText;
                if (dateTime != null) {
                    if (line2Text != null)
                        line2Text += ", " + dateTime;
                    else
                        line2Text = dateTime;
                }
                numSections++;
                drawerSections.push(labeledValue(activityText, null) + "<br/>"
                                  + labeledValue(null, line2Text));
            }
        }
        setDrawerContents();
    }
    catch (ex) {
        // Exception; presumably the HTML structure wasn't what we expected.
        // Probably bad tracking number or the page layout changed.
        closeDrawer();
        resetDisplay();
        setMessage("widget-status", "Couldn't read package status.");
        setMessage("error", "Tracking number may be incorrect, or the tracking site layout changed.");
	    //alert("DHL FAIL: " + ex);
    }
}
DHLTracker.prototype.scrapeHtml = DHLTracker_scrapeHtml;
